﻿// SampleConfig.cs
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the code for an exemplary set of classes that demonstrate (and also test and prove) the
// capabilities of the XMLConfig library.

#region Using declarations

using System;
using System.Collections.Generic;
using System.Drawing;

#endregion

namespace XMLConfig {

  // SampleConfig: This class is a container for a list of sample classes to be written to an XMLConfig file.

  public class SampleConfig : IWritable {

    #region Variables and initialization code

    public List<SampleClass> lstSampleClasses = new List<SampleClass>();

    #endregion

    #region XMLConfig:IWritable interface code

    public SampleConfig() { }

    public virtual void Initialize(Dictionary<string, object> dObjectMembers) {
      lstSampleClasses = (List<SampleClass>) dObjectMembers["lstSampleClasses"];
    }

    public void WriteConfig(XMLConfigWriter writer) {
      writer.WriteObject("lstSampleClasses", lstSampleClasses);
    }

    #endregion

    #region Helper Classes

    public void Read(string strFilename) {
      SampleConfig sc = (SampleConfig) XMLConfigReader.Read(strFilename);
      if (sc != null)
        this.lstSampleClasses = sc.lstSampleClasses;
    }

    public static string GetRandomString(Random r) {
      string[] astrAdjectives = new string[] { "blue", "yellow", "plaid", "happy", "sad", "mellow", "gigantic", "miniature", "dancing", "skating" };
      string[] astrNouns = new string[] { "turtles", "whales", "manatees", "narwhals", "penguins", "walruses", "lions", "zebras", "kittens", "rabbits" };
      return (r.Next(100) + 1) + " " + astrAdjectives[r.Next(10)] + " " + astrNouns[r.Next(10)];
    }

    #endregion

  }

  // SampleConfig: This class is the base class from which other classes are derived.
  // Note that every class includes an Enum type that defines the type of the class
  // instance (and also demonstrates that XMLConfig handles Enums well and natively).

  public abstract class SampleClass : IWritable {

    #region Variables and initialization code

    public string strName = "";
    public SampleClassTypeEnum enumSampleClassType;

    public SampleClass(string strName) {
      this.strName = strName;
    }

    #endregion

    #region XMLConfig:IWritable interface code

    public SampleClass() { }

    public virtual void Initialize(Dictionary<string, object> dObjectMembers) {
      enumSampleClassType = (SampleClassTypeEnum) dObjectMembers["enumSampleClassType"];
      strName = (string) dObjectMembers["strName"];
    }

    public virtual void WriteConfig(XMLConfigWriter writer) {
      writer.WriteObject("enumSampleClassType", enumSampleClassType);
      writer.WriteObject("strName", strName);
    }

    #endregion

  }

  // DerivedClass1: This class is a simple derivation of SampleClass, with nothing added.

  public class DerivedClass1 : SampleClass {

    #region Variables and initialization code

    public DerivedClass1(string strName) : base(strName) {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass1;
    }

    #endregion

    #region XMLConfig:IWritable interface code

    public DerivedClass1() : base() {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass1;
    }

    public override void Initialize(Dictionary<string, object> dObjectMembers) {
      base.Initialize(dObjectMembers);
    }

    public override void WriteConfig(XMLConfigWriter writer) {
      base.WriteConfig(writer);
    }

    #endregion

  }

  // DerivedClass2: This class is a derivation of SampleClass with a bunch of simple
  // members added, including many primitive types and a member instance of the
  // DerivedClass1 class.

  public class DerivedClass2 : SampleClass {

    #region Variables and initialization code

    public string strDerivedClass2Property1 = "This & is < my > sample % string";                   // just a string
    public int iDerivedClass2Property2 = 9;                                                         // just an int
    public bool bDerivedClass2Property3 = true;                                                     // just a bool
    public DateTime dtDerivedClass2Property4 = DateTime.Now;                                        // just a DateTime
    public long lDerivedClass2Property5 = 12;                                                       // just a long
    public float fDerivedClass2Property6 = 3.14f;                                                   // just a float
    public double dDerivedClass2Property7 = 3.14;                                                   // just a double
    private string strDerivedClass2Property8 = "Secret Message!";                                   // a private string that is encrypted and decrypted,
    public string GetString8() { return strDerivedClass2Property8; }                                // and accessed through an accessor
    public DerivedClass1 oDerivedClass2Property9 = new DerivedClass1("Nested DerivedClass1");       // a nested memer class

    public DerivedClass2(string strName, Random r) : base(strName) {
      this.strName = strName;
      iDerivedClass2Property2 = r.Next(Int32.MaxValue);
      lDerivedClass2Property5 = (long) (r.Next(Int32.MaxValue) * 1000000);
      fDerivedClass2Property6 = (float) (r.Next(Int32.MaxValue) / 10000.0f);
      dDerivedClass2Property7 = (double) (r.Next(Int32.MaxValue) / 10000.0f);
      enumSampleClassType = SampleClassTypeEnum.DerivedClass2;
    }

    #endregion

    #region XMLConfig:IWritable interface code

    public DerivedClass2() : base() {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass2;
    }

    public override void Initialize(Dictionary<string, object> dObjectMembers) {
      base.Initialize(dObjectMembers);
      strDerivedClass2Property1 = (string) dObjectMembers["strDerivedClass2Property1"];
      iDerivedClass2Property2 = (int) dObjectMembers["iDerivedClass2Property2"];
      bDerivedClass2Property3 = (bool) dObjectMembers["bDerivedClass2Property3"];
      dtDerivedClass2Property4 = (DateTime) dObjectMembers["dtDerivedClass2Property4"];
      lDerivedClass2Property5 = (long) dObjectMembers["lDerivedClass2Property5"];
      fDerivedClass2Property6 = (float) dObjectMembers["fDerivedClass2Property6"];
      dDerivedClass2Property7 = (double) dObjectMembers["dDerivedClass2Property7"];
      strDerivedClass2Property8 = XMLConfigReader.DecryptString((string) dObjectMembers["strDerivedClass2Property8"], "MyPassword");
      oDerivedClass2Property9 = (DerivedClass1) dObjectMembers["oDerivedClass2Property9"];
    }

    public override void WriteConfig(XMLConfigWriter writer) {
      base.WriteConfig(writer);
      writer.WriteObject("strDerivedClass2Property1", strDerivedClass2Property1);
      writer.WriteObject("iDerivedClass2Property2", iDerivedClass2Property2);
      writer.WriteObject("bDerivedClass2Property3", bDerivedClass2Property3);
      writer.WriteObject("dtDerivedClass2Property4", dtDerivedClass2Property4);
      writer.WriteObject("lDerivedClass2Property5", lDerivedClass2Property5);
      writer.WriteObject("fDerivedClass2Property6", fDerivedClass2Property6);
      writer.WriteObject("dDerivedClass2Property7", dDerivedClass2Property7);
      writer.WriteObject("strDerivedClass2Property8", XMLConfigWriter.EncryptString(strDerivedClass2Property8, "MyPassword"));
      writer.WriteObject("oDerivedClass2Property9", oDerivedClass2Property9);
    }

    #endregion

  }

  // DerivedClass3: This class is a derivation of SampleClass with some collections,
  // including a List of strings, an array of ints, a Dictionary, and a Bitmap (which
  // is serialized as a binary object).

  public class DerivedClass3 : SampleClass {

    #region Variables and initialization code

    public List<string> lstStrings = new List<string>();
    public int[] aInts = new int[] { };
    public Dictionary<string, object> dParameters = new Dictionary<string, object>();
    public Bitmap bitmap = new Bitmap(120, 120);

    public DerivedClass3(string strName, Random r) : base(strName) {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass3;
      this.strName = strName;
      lstStrings.Add("Sample String 1: " + SampleConfig.GetRandomString(r));
      lstStrings.Add("Sample String 2: " + SampleConfig.GetRandomString(r));
      lstStrings.Add("Sample String 3: " + SampleConfig.GetRandomString(r));
      aInts = new int[] { r.Next(Int32.MaxValue), r.Next(Int32.MaxValue), r.Next(Int32.MaxValue) };
      dParameters.Add("Sample Key 1", (int) r.Next(Int32.MaxValue));
      dParameters.Add("Sample Key 2", SampleConfig.GetRandomString(r));
      dParameters.Add("Sample Key 3", r.Next(2) == 0 ? true : false);
      bitmap = new Bitmap(120, 120);
      Graphics g = Graphics.FromImage(bitmap);
      g.FillRectangle(Brushes.Black, new Rectangle(0, 0, 120, 120));
      for (int i = 0; i < 4; i++) {
        Brush brush = new SolidBrush(Color.FromArgb(r.Next(256), r.Next(256), r.Next(256)));
        if (r.Next(2) == 1)
          g.FillRectangle(brush, new Rectangle(r.Next(90), r.Next(90), r.Next(20) + 10, r.Next(20) + 10));
        else
          g.FillEllipse(brush, new Rectangle(r.Next(90), r.Next(90), r.Next(20) + 10, r.Next(20) + 10));
      }
    }

    #endregion

    #region XMLConfig:IWritable interface code

    public DerivedClass3() : base() {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass3;
    }

    public override void Initialize(Dictionary<string, object> dObjectMembers) {
      base.Initialize(dObjectMembers);
      lstStrings = (List<string>) dObjectMembers["lstStrings"];
      aInts = (int[]) dObjectMembers["aInts"];
      dParameters = (Dictionary<string, object>) dObjectMembers["dParameters"];
      bitmap = (Bitmap) dObjectMembers["bitmap"];
    }

    public override void WriteConfig(XMLConfigWriter writer) {
      base.WriteConfig(writer);
      writer.WriteObject("lstStrings", lstStrings);
      writer.WriteObject("aInts", aInts);
      writer.WriteObject("dParameters", dParameters);
      writer.WriteObject("bitmap", bitmap);
    }

    #endregion

  }

  // DerivedClass4: This class is a derivation of SampleClass with references to
  // other class instances that form non-hierarchical organizations in the
  // class list (a self-reference, a reference to a peer that references the
  // same class, and a reference to a child that is also referenced by the peer).
  // This class demonstrates (and tests and proves) the capability of XMLConfig
  // to handle non-hierarchically-structured object sets.

  public class DerivedClass4 : SampleClass {

    #region Variables and initialization code

    public SampleClass scSelf = null, scPeer = null, scChild = null;

    public DerivedClass4(string strName) : base(strName) {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass4;
      this.strName = strName;
      scSelf = this;
      scChild = new DerivedClass1("Child Object");
      scPeer = new DerivedClass4("Peer Object", this, scChild);
    }

    // note: this constructor is used to instantiate a DerivedClass4 instance that is a peer of an already existing DerivedClass4 instance.
    public DerivedClass4(string strName, SampleClass scParent, SampleClass scChild) : base(strName) {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass4;
      this.strName = strName;
      this.scSelf = this;
      this.scChild = scChild;
      this.scPeer = scParent;
    }

    #endregion

    #region XMLConfig:IWritable interface code

    public DerivedClass4() : base() {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass4;
      scSelf = this;
      scChild = new DerivedClass1("Child Object");
      scPeer = new DerivedClass4("Peer Object", this, scChild);
    }

    public override void Initialize(Dictionary<string, object> dObjectMembers) {
      base.Initialize(dObjectMembers);
      scSelf = (SampleClass) dObjectMembers["scSelf"];
      scChild = (SampleClass) dObjectMembers["scChild"];
      scPeer = (SampleClass) dObjectMembers["scPeer"];
    }

    public override void WriteConfig(XMLConfigWriter writer) {
      base.WriteConfig(writer);
      writer.WriteObject("scSelf", scSelf);
      writer.WriteObject("scChild", scChild);
      writer.WriteObject("scPeer", scPeer);
    }

    #endregion

  }

  // DerivedClass4: This class is a derivation of SampleClass with various
  // types of nulls to demonstrate (and test and prove) the ability of
  // XMLConfig to handle nulls in a graceful manner.

  public class DerivedClass5 : SampleClass {

    #region Variables and initialization code

    public int[] arrayNull = null;  // remains a null reference
    public List<object> lstNull = new List<object>();  // contains a null list entry
    public Dictionary<object, object> dNull = new Dictionary<object, object>();  // contains a null dictionary value (NOTE: null keys are disallowed by the Dictionary class)

    public DerivedClass5(string strName) : base(strName) {
      enumSampleClassType = SampleClassTypeEnum.DerivedClass5;
      this.lstNull.Add(null);
      this.dNull.Add("Null Value", null);
    }

    #endregion

    #region XMLConfig:IWritable interface code

    public DerivedClass5() : base() {
      this.lstNull.Add(null);
      this.dNull.Add("Null Value", null);
    }

    public override void Initialize(Dictionary<string, object> dObjectMembers) {
      base.Initialize(dObjectMembers);
      arrayNull = (int[]) dObjectMembers["arrayNull"];
      lstNull = (List<object>) dObjectMembers["lstNull"];
      dNull = (Dictionary<object, object>) dObjectMembers["dNull"];
    }

    public override void WriteConfig(XMLConfigWriter writer) {
      base.WriteConfig(writer);
      writer.WriteObject("arrayNull", arrayNull);
      writer.WriteObject("lstNull", lstNull);
      writer.WriteObject("dNull", dNull);
    }

    #endregion

  }

  #region Helper Classes

  public enum SampleClassTypeEnum { DerivedClass1, DerivedClass2, DerivedClass3, DerivedClass4, DerivedClass5 };

  #endregion

}
