﻿// XMLConfigDemo.cs
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the code for an application that demonstrates the application of the XMLConfig API to
// write and read various sets of objects created by the user.

#region Using declarations

using System;
using System.Drawing;
using System.Windows.Forms;

#endregion

namespace XMLConfig {

  public partial class XMLConfigDemo : Form {

    #region variables and initialization code

    public Random r = new Random((int) DateTime.Now.Ticks % Int32.MaxValue);
    public SampleConfig configSample = null;

    [STAThread]
    static void Main() {
      Application.EnableVisualStyles();
      Application.SetCompatibleTextRenderingDefault(false);
      Application.Run(new XMLConfigDemo());
    }
    
    public XMLConfigDemo() {
      InitializeComponent();
    }

    #endregion

    #region Event handlers

    private void XMLConfigDemo_Load(object sender, EventArgs e) {
      LoadConfigTreeview();
    }

    private void btnCreateSampleConfig_Click(object sender, EventArgs e) {
      configSample = new SampleConfig();
      LoadConfigTreeview();
    }

    private void btnEraseSampleConfig_Click(object sender, EventArgs e) {
      configSample = null;
      LoadConfigTreeview();
    }

    private void btnSave_Click(object sender, EventArgs e) {
      SaveFileDialog sfd = new SaveFileDialog();
      sfd.Filter = "Config file (*.cfg)|*.cfg";
      if (sfd.ShowDialog() == DialogResult.OK)
        XMLConfigWriter.Write(sfd.FileName, configSample);
    }

    private void btnLoad_Click(object sender, EventArgs e) {
      OpenFileDialog ofd = new OpenFileDialog();
      ofd.Filter = "Config file (*.cfg)|*.cfg";
      if (ofd.ShowDialog() == DialogResult.OK) {
        object o = XMLConfigReader.Read(ofd.FileName);
        LoadConfigTreeview();
      }
    }

    private void btnAddClass1_Click(object sender, EventArgs e) {
      if (configSample != null) {
        configSample.lstSampleClasses.Add(new DerivedClass1("Object " + (configSample.lstSampleClasses.Count + 1)));
        LoadConfigTreeview();
      }
    }

    private void btnAddClass2_Click(object sender, EventArgs e) {
      if (configSample != null) {
        configSample.lstSampleClasses.Add(new DerivedClass2("Object " + (configSample.lstSampleClasses.Count + 1), r));
        LoadConfigTreeview();
      }
    }

    private void btnAddClass3_Click(object sender, EventArgs e) {
      if (configSample != null) {
        configSample.lstSampleClasses.Add(new DerivedClass3("Object " + (configSample.lstSampleClasses.Count + 1), r));
        LoadConfigTreeview();
      }
    }

    private void btnAddClass4_Click(object sender, EventArgs e) {
      if (configSample != null) {
        configSample.lstSampleClasses.Add(new DerivedClass1("Object " + (configSample.lstSampleClasses.Count + 1)));
        LoadConfigTreeview();
      }
    }

    private void btnAddClass5_Click(object sender, EventArgs e) {
      if (configSample != null) {
        configSample.lstSampleClasses.Add(new DerivedClass1("Object " + (configSample.lstSampleClasses.Count + 1)));
        LoadConfigTreeview();
      }
    }

    private void tvConfigDemo_DoubleClick(object sender, EventArgs e) {
      if ((tvConfigDemo.SelectedNode != null) && (tvConfigDemo.SelectedNode.Tag != null))
        new DisplayBitmap((Bitmap) tvConfigDemo.SelectedNode.Tag).ShowDialog();
    }

    #endregion

    #region Helper classes

    private void LoadConfigTreeview() {
      tvConfigDemo.BeginUpdate();
      tvConfigDemo.Nodes.Clear();
      if (configSample != null) {
        TreeNode tn = tvConfigDemo.Nodes.Add("SampleConfig");
        TreeNode tnList = tn.Nodes.Add("lstSampleClasses");
        foreach (SampleClass sc in configSample.lstSampleClasses)
          LoadConfigTreeview(sc, tnList);
      }
      tvConfigDemo.ExpandAll();
      tvConfigDemo.EndUpdate();
      tvConfigDemo.Refresh();
    }

    private void LoadConfigTreeview(SampleClass sc, TreeNode tnParent) {
      TreeNode tnSampleClass = tnParent.Nodes.Add("SampleClass");
      tnSampleClass.Tag = sc;
      tnSampleClass.Nodes.Add("Name: " + sc.strName);
      tnSampleClass.Nodes.Add("Type: " + sc.enumSampleClassType.ToString());
      switch (sc.enumSampleClassType) {
        case SampleClassTypeEnum.DerivedClass2:
          DerivedClass2 sc2 = (DerivedClass2) sc;
          tnSampleClass.Nodes.Add("Property 1 (string): " + sc2.strDerivedClass2Property1);
          tnSampleClass.Nodes.Add("Property 2 (int): " + sc2.iDerivedClass2Property2);
          tnSampleClass.Nodes.Add("Property 3 (bool): " + sc2.bDerivedClass2Property3);
          tnSampleClass.Nodes.Add("Property 4 (DateTime): " + sc2.dtDerivedClass2Property4.ToShortDateString());
          tnSampleClass.Nodes.Add("Property 5 (long): " + sc2.lDerivedClass2Property5);
          tnSampleClass.Nodes.Add("Property 6 (float): " + sc2.fDerivedClass2Property6);
          tnSampleClass.Nodes.Add("Property 7 (double): " + sc2.dDerivedClass2Property7);
          tnSampleClass.Nodes.Add("Property 8 (encrypted string): " + sc2.GetString8());
          TreeNode tnNestedClass = tnSampleClass.Nodes.Add("Property 9 (nested object)");
          LoadConfigTreeview(sc2.oDerivedClass2Property9, tnNestedClass);
          break;
        case SampleClassTypeEnum.DerivedClass3:
          DerivedClass3 sc3 = (DerivedClass3) sc;
          TreeNode tnContainer = tnSampleClass.Nodes.Add("lstStrings");
          foreach (string s in sc3.lstStrings)
            tnContainer.Nodes.Add("String: " + s);
          tnContainer = tnSampleClass.Nodes.Add("aInts");
          foreach (int i in sc3.aInts)
            tnContainer.Nodes.Add("Int: " + i);
          tnContainer = tnSampleClass.Nodes.Add("Dictionary");
          foreach (string strKey in sc3.dParameters.Keys) {
            object o = sc3.dParameters[strKey];
            tnContainer.Nodes.Add(strKey + ": " + o.GetType().FullName + ": " + o.ToString());
          }
          TreeNode tnBitmap = tnSampleClass.Nodes.Add("Bitmap: " + (sc3.bitmap == null ? "null" : "loaded [double-click this node to view]"));
          tnBitmap.Tag = sc3.bitmap;
          break;
        case SampleClassTypeEnum.DerivedClass4:
          DerivedClass4 sc4 = (DerivedClass4) sc;
          tnSampleClass.Nodes.Add("Self Reference: " + (sc4.scSelf == sc4));
          if (sc4.strName == "Peer Object") {
            tnSampleClass.Nodes.Add("Peer Reference: " + (sc4.scPeer == tnSampleClass.Parent.Parent.Tag ? "[back to parent]" : "[elsehwere?]"));
            tnSampleClass.Nodes.Add("Child Reference: " + (sc4.scChild == ((DerivedClass4) tnSampleClass.Parent.Parent.Tag).scChild ? "[same as peer]" : "[different from peer]"));
          } else {
            TreeNode tnChildNode = tnSampleClass.Nodes.Add("Child Reference");
            LoadConfigTreeview(sc4.scChild, tnChildNode);
            TreeNode tnPeerNode = tnSampleClass.Nodes.Add("Peer Reference");
            LoadConfigTreeview(sc4.scPeer, tnPeerNode);
          }
          break;
        case SampleClassTypeEnum.DerivedClass5:
          DerivedClass5 sc5 = (DerivedClass5) sc;
          tnSampleClass.Nodes.Add("arrayNull value: " + (sc5.arrayNull == null ? "<null>" : "<something>"));
          tnContainer = tnSampleClass.Nodes.Add("lstNull");
          foreach (object o in sc5.lstNull)
            tnContainer.Nodes.Add("Value: " + (o == null? "<null>" : o.ToString()));
          tnContainer = tnSampleClass.Nodes.Add("dNull");
          foreach (object o in sc5.dNull.Keys) {
            tnContainer.Nodes.Add("Key: " + o.ToString());
            tnContainer.Nodes.Add("Value: " + (sc5.dNull[o] == null? "<null>" : sc5.dNull[o].ToString()));
          }
          break;
        default:
          break;
      }
    }

    #endregion

  }
}
