﻿// XMLConfigReader.cs
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains all of the code for reading an object set from an XMLConfig file.

#region Using declarations

using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Xml;
using System.Reflection;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

#endregion

namespace XMLConfig {

  public class XMLConfigReader {

    #region Variables and Initialization Code

    public XmlReader reader = null;
    private Dictionary<string, Type> dFoundTypes = new Dictionary<string, Type>();
    private Dictionary<int, object> dObjectIDs = new Dictionary<int, object>();
    private List<Reference> lstReferencesToPatch = new List<Reference>();
    private Dictionary<object, Dictionary<string, object>> dObjectsToInitialize = new Dictionary<object, Dictionary<string, object>>();
    private object oReturnObject = null;  // the object to be returned following a Read call

    public XMLConfigReader(string strFilename) {
      // read up to and past the "Objects" node
      reader = XmlReader.Create(strFilename);
    }

    #endregion

    #region Main Functions

    /// <summary>
    /// Read the XMLConfig file at the specified location and returns the top-level object.
    /// </summary>
    /// <param name="strFilename">The filename of the XMLConfig file.</param>
    public static object Read(string strFilename) {
      try {
        XMLConfigReader xmlConfig = new XMLConfigReader(strFilename);
        // throw away node types until we reach the Objects object
        while ((xmlConfig.reader.ReadState != ReadState.EndOfFile) && (xmlConfig.reader.Name != "Objects"))
          xmlConfig.reader.Read();
        if (xmlConfig.reader.ReadState == ReadState.EndOfFile)
          return null;
        xmlConfig.reader.Read();  // read past Objects node
        while ((xmlConfig.reader.ReadState != ReadState.EndOfFile) && (xmlConfig.reader.Name != "Objects"))
          xmlConfig.ReadObject();  // read objects until reaching the end-of-objects tag
        xmlConfig.reader.Close();
        
        // reading has concluded - fix up and initialize objects
        foreach (Reference r in xmlConfig.lstReferencesToPatch)
          r.Fixup(xmlConfig.dObjectIDs);
        xmlConfig.InitializeObjects();
        return xmlConfig.oReturnObject;
      } catch (Exception ex) {
        throw new Exception("Error reading config: " + ex.Message + " See InnerException for more info.", ex);
      }
    }

    private void InitializeObjects() {
      // This function initializes each object with its dictionary set.
      // For Lists, Arrays (handled as Lists), and Dictionaries, instantiation involves deserializing the members into the collection.
      // For IWritable objects, initialization involves calling the Initialize() function with the member dictionary of the object.
      foreach (object o in dObjectsToInitialize.Keys) {
        Dictionary<string, object> dObjectMembers = dObjectsToInitialize[o];
        if (o is IDictionary) {
          IDictionary dictionary = (IDictionary) o;
          for (int i = 0; i < dObjectMembers.Keys.Count / 2; i++) {
            object oKey = dObjectMembers["Key " + i];
            object oValue = dObjectMembers["Value " + i];
            dictionary.Add(oKey, oValue);
          }
        } else if (o is IList) {
          IList list = (IList) o;
          for (int i = 0; i < dObjectMembers.Keys.Count; i++) {
            if (list.IsFixedSize)
              list[i] = dObjectMembers[i.ToString()];
            else
              list.Insert(i, dObjectMembers[i.ToString()]);
          }
        } else if (o is IWritable) {
          IWritable writable = (IWritable) o;
          writable.Initialize(dObjectsToInitialize[o]);
        }
      }
    }

    private bool ReadObject() {
      // This function reads the object having a (start) node at the current file pointer of the XMLReader.
      //    Instantiate the object
      //    Add to read-objects list, and if it's the first object read, remember to return it

      if (reader != null) {
        try {
          while (reader.NodeType != XmlNodeType.Element) {
            if (reader.Read() == false)
              return false;
          }

          // read basic info about object
          string strType = DecodeString(reader.Name);
          if (strType == "null") {
            reader.Read();
            return true;
          }
          
          string strSubtype = "";
          string strID = "";
          string strValue = "";
          
          if ((strType == "List") || (strType == "Array") || (strType == "Dictionary") || (strType == "Object")) {
            strSubtype = ReadAttribute("Type");
            strID = ReadAttribute("ID");
          } else {
            if (strType == "Enum")
              strSubtype = ReadAttribute("Type");
            strValue = ReadAttribute("Value");
          }

          // if object has members, read them (and register any references for fixup after reading and instantiating all objects)
          Dictionary<string, object> dObjectMembers = new Dictionary<string, object>();
          while ((reader.Read()) && (reader.NodeType != XmlNodeType.EndElement)) {
            if (reader.NodeType == XmlNodeType.Element) {
              string strMemberType = DecodeString(reader.Name);
              string strMemberName = "";
              if ((strType == "List") || (strType == "Array"))
                strMemberName = dObjectMembers.Count.ToString();
              else if (strType == "Dictionary")
                strMemberName = (dObjectMembers.Count % 2 == 0 ? "Key " : "Value ") + dObjectMembers.Count / 2;
              else
                strMemberName = ReadAttribute("Name");
              if (strMemberType == "Reference") {
                int iMemberID = Int32.Parse(reader.GetAttribute("ID"));
                dObjectMembers.Add(strMemberName, iMemberID);
                lstReferencesToPatch.Add(new Reference(dObjectMembers, strMemberName));
              } else {  // value type - just decode and add
                string strMemberValue = (strMemberType == "Null" ? "" : ReadAttribute("Value"));
                string strMemberSubtype = (strMemberType == "Enum" ? ReadAttribute("Type") : "");
                object oMemberValue = null;
                if (ConvertKnownTypes(strMemberType, strMemberSubtype, strMemberValue, ref oMemberValue))
                  dObjectMembers.Add(strMemberName, oMemberValue);
              }
            } else if (reader.NodeType == XmlNodeType.CDATA)
                dObjectMembers.Add("CDATA", BinaryDeserialize(reader.Value));
          }

          // instantiate the object
          object oCurrentObject = null;
          switch (strType) {
            case "List":
              oCurrentObject = InstantiateList(strSubtype);
              dObjectsToInitialize.Add(oCurrentObject, dObjectMembers);
              break;
            case "Array":
              oCurrentObject = InstantiateArray(strSubtype, dObjectMembers.Count);
              dObjectsToInitialize.Add(oCurrentObject, dObjectMembers);
              break;
            case "Dictionary":
              oCurrentObject = InstantiateDictionary(strSubtype);
              dObjectsToInitialize.Add(oCurrentObject, dObjectMembers);
              break;
            case "Object":  // try getting parameterless constructor
              if (strSubtype == "BinaryObject")
                oCurrentObject = dObjectMembers["CDATA"];
              else {
                Type typeNewObject = GetObjectType(strSubtype);
                if (typeNewObject == null)
                  throw new Exception("No parameterless constructor for class of type " + strType + ".");
                ConstructorInfo ci = typeNewObject.GetConstructor(new Type[] { });
                if (ci == null)
                  throw new Exception("Invocation of parameterless constructor for class of type " + strType + " failed.");
                oCurrentObject = ci.Invoke(new object[] { });
                dObjectsToInitialize.Add(oCurrentObject, dObjectMembers);
              }
              break;
            default:
              if (ConvertKnownTypes(strType, strSubtype, strValue, ref oCurrentObject) == false)
                throw new Exception("Couldn't figure out how to instantiate type " + strType + (strSubtype.Length > 0 ? ": " + strSubtype : "") + ".");
              break;
          }

          // add to read objects list
          if (strID.Length > 0)
            dObjectIDs.Add(Int32.Parse(strID), oCurrentObject);
          if (oReturnObject == null)
            oReturnObject = oCurrentObject;
          return true;
        } catch (Exception ex) {
          MessageBox.Show("Error reading configuration file: " + ex.Message);
        }
      }
      return false;
    }

    #endregion

    #region Helper Functions

    private object InstantiateList(string strSubtype) {
      return (IList) Activator.CreateInstance(typeof(List<>).MakeGenericType(GetObjectType(strSubtype)));
    }

    private object InstantiateArray(string strSubtype, int iLength) {
      return Array.CreateInstance(GetObjectType(strSubtype), iLength);
    }

    private object InstantiateDictionary(string strSubtype) {
      string[] strSubtypes = strSubtype.Split(',');
      Type typeKey = GetObjectType(strSubtypes[0]);
      Type typeValue = GetObjectType(strSubtypes[1]);
      IDictionary d = (IDictionary) Activator.CreateInstance(typeof(Dictionary<,>).MakeGenericType(typeKey, typeValue));
      return d;
    }

    private Type GetObjectType(string strTypeName) {
      if (dFoundTypes.ContainsKey(strTypeName))
        return dFoundTypes[strTypeName];
      Type t = Type.GetType(strTypeName);
      if (t != null) {
        dFoundTypes.Add(strTypeName, t);
        return t;
      }
      // this type isn't defined in mscorlib or XMLConfig - try retrieving from all loaded assemblies.
      Assembly[] aAssemblies = AppDomain.CurrentDomain.GetAssemblies();
      foreach (Assembly a in aAssemblies) {
        Type[] aTypes = a.GetTypes();
        foreach (Type type in aTypes) {
          if (type.FullName == strTypeName) {
            dFoundTypes.Add(strTypeName, type);
            return type;
          }
        }
      }
      return null;
    }

    private static object BinaryDeserialize(string strBinary) {
      try {
        MemoryStream memoryStream = new MemoryStream(System.Convert.FromBase64String(strBinary));
        return new BinaryFormatter().Deserialize(memoryStream);
      } catch (Exception ex) {
        MessageBox.Show("Error deserializing object: " + ex.Message);
        return "";
      }
    }

    private bool ConvertKnownTypes(string strType, string strSubtype, string strValue, ref object oValue) {
      switch (strType) {
        case "System.String": oValue = strValue; return true;
        case "System.Boolean": oValue = Boolean.Parse(strValue); return true;
        case "System.Char": oValue = Char.Parse(strValue); return true;
        case "System.SByte": oValue = SByte.Parse(strValue); return true;
        case "System.Byte": oValue = Byte.Parse(strValue); return true;
        case "System.Int16": oValue = Int16.Parse(strValue); return true;
        case "System.UInt16": oValue = UInt16.Parse(strValue); return true;
        case "System.Int32": oValue = Int32.Parse(strValue); return true;
        case "System.UInt32": oValue = UInt32.Parse(strValue); return true;
        case "System.Int64": oValue = Int64.Parse(strValue); return true;
        case "System.UInt64": oValue = UInt64.Parse(strValue); return true;
        case "System.Single": oValue = Single.Parse(strValue); return true;
        case "System.Double": oValue = Double.Parse(strValue); return true;
        case "System.Decimal": oValue = Decimal.Parse(strValue); return true;
        case "System.DateTime": oValue = DateTime.Parse(strValue); return true;
        case "Enum": oValue = Enum.Parse(GetObjectType(strSubtype), strValue); return true;
        case "null": oValue = null; return true;
        default: return false;
      }
    }

    /// <summary>
    /// Decrypts the specified encrypted string with the specified password using the Rijndael encryption method.
    /// </summary>
    /// <param name="strEncryptedString">The encrypted string.</param>
    /// <param name="strPassword">The password with which the string was encrypted.</param>
    public static string DecryptString(string strEncryptedString, string strPassword) {
      try {
        byte[] bCipherBytes = Convert.FromBase64String(strEncryptedString);
        System.Security.Cryptography.PasswordDeriveBytes pdb = new System.Security.Cryptography.PasswordDeriveBytes(strPassword, new byte[] { 0x49, 0x76, 0x61, 0x6e, 0x20, 0x4d, 0x65, 0x64, 0x76, 0x65, 0x64, 0x65, 0x76 });
        System.IO.MemoryStream ms = new System.IO.MemoryStream();
        System.Security.Cryptography.Rijndael alg = System.Security.Cryptography.Rijndael.Create();
        alg.Key = pdb.GetBytes(32);
        alg.IV = pdb.GetBytes(16);
        System.Security.Cryptography.CryptoStream cs = new System.Security.Cryptography.CryptoStream(ms, alg.CreateDecryptor(), System.Security.Cryptography.CryptoStreamMode.Write);
        cs.Write(bCipherBytes, 0, bCipherBytes.Length);
        cs.Close();
        byte[] bData = ms.ToArray();
        if (bData == null)
          return "";
        else
          return System.Text.Encoding.Unicode.GetString(bData);
      } catch {
        return "";
      }
    }

    private string ReadAttribute(string strAttribute) {
      return System.Web.HttpUtility.HtmlDecode((string) reader.GetAttribute(strAttribute));
    }

    private string DecodeString(string s) {
      return System.Web.HttpUtility.HtmlDecode(s);
    }

    #endregion

  }

  #region Helper Classes

  public interface IWritable {
    void WriteConfig(XMLConfigWriter writer);
    void Initialize(Dictionary<string, object> dObjectMembers);
  }

  public class Reference {
    public IDictionary dictionary;
    public string strKey;
    public Reference(IDictionary dictionary, string strKey) {
      this.dictionary = dictionary;
      this.strKey = strKey;
    }
    public void Fixup(Dictionary<int, object> dReadObjects) {
      int iID = (int) dictionary[strKey];
      dictionary[strKey] = dReadObjects[iID];
    }
  }

  #endregion

}
