﻿// XMLWriterReader.cs
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains all of the code for writing an object set to an XMLConfig file.

#region Using Declarations

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

#endregion

namespace XMLConfig {

  public class XMLConfigWriter {

    #region Variables and Initialization Code

    public XmlWriter writer = null;
    public Dictionary<object, int> dObjectIDs = new Dictionary<object, int>();
    public List<object> lstWriteQueue = new List<object>();

    public XMLConfigWriter(string strFilename, string strEncoding = "Unicode", bool bAddWhitespace = true) {
      XmlWriterSettings settings = new XmlWriterSettings();
      settings.Encoding = Encoding.GetEncoding(strEncoding);
      settings.Indent = bAddWhitespace;
      settings.IndentChars = (bAddWhitespace ? "  " : "");
      settings.NewLineOnAttributes = false;
      writer = XmlWriter.Create(strFilename, settings);
      writer.WriteStartDocument();
    }

    #endregion

    #region Main Functions

    /// <summary>
    /// Write the provided object to an XMLConfig file at the specified location.
    /// </summary>
    /// <param name="strFilename">The filename of the XMLConfig file.</param>
    public static void Write(string strFilename, object o) {
      try {
        XMLConfigWriter writer = new XMLConfigWriter(strFilename);
        writer.writer.WriteStartElement("Objects");
        writer.lstWriteQueue.Add(o);
        if ((o != null) && (o.GetType().IsPrimitive == false))
          writer.dObjectIDs.Add(o, 0);
        while (writer.lstWriteQueue.Count > 0) {
          writer.WriteObject(writer.lstWriteQueue[0], "", true);
          writer.lstWriteQueue.RemoveAt(0);
        }
        writer.writer.WriteEndElement();
        writer.Close();
      } catch (Exception ex) {
        throw new Exception("Error writing config: " + ex.Message + " See InnerException for more info.", ex);
      }
    }

    private void WriteObject(object o, string strName = "", bool bWrite = false) {
      // This function writes an object to an XMLConfig file.  This function is (slightly) recursive: if the object
      // is to be written (bWrite = true), it writes all of its members, and non-primitive members are written as
      // references by calling WriteObject with bWrite = false.  (Also, when a reference is written for an object
      // that does not yet have an ID, it is assigned an ID and enqueued for writing.)

      try {
        if (o == null) {
          writer.WriteStartElement("null");
          if (strName.Length > 0)
            writer.WriteAttributeString("Name", EncodeString(strName));
          writer.WriteEndElement();
        } else if (o is Enum) {
          writer.WriteStartElement("Enum");
          if (strName.Length > 0)
            writer.WriteAttributeString("Name", EncodeString(strName));
          writer.WriteAttributeString("Type", EncodeString(o.GetType().FullName));
          writer.WriteAttributeString("Value", EncodeString(o.ToString()));
          writer.WriteEndElement();
        } else if ((o.GetType().IsValueType) || (o is string)) {
          writer.WriteStartElement(o.GetType().FullName);
          if (strName.Length > 0)
            writer.WriteAttributeString("Name", EncodeString(strName));
          writer.WriteAttributeString("Value", EncodeString(o.ToString()));
          writer.WriteEndElement();
        } else {  // this is an object of some type
          if (bWrite) {
            // generate an ID if it doesn't already have one
            if (dObjectIDs.ContainsKey(o) == false) {  // enqueue for writing
              int iID = dObjectIDs.Keys.Count;
              dObjectIDs.Add(o, iID);
            }
            if (o is Array)
              WriteArray((IList) o);
            else if (o is IList)
              WriteList((IList) o);
            else if (o is IDictionary)
              WriteDictionary((IDictionary) o);
            else {
              writer.WriteStartElement("Object");
              writer.WriteAttributeString("ID", dObjectIDs[o].ToString());
              if (o is IWritable) {
                writer.WriteAttributeString("Type", EncodeString(o.GetType().FullName));
                ((IWritable) o).WriteConfig(this);
              } else if (o.GetType().IsSerializable) {
                writer.WriteAttributeString("Type", "BinaryObject");
                writer.WriteCData(BinarySerialize(o));
              } else  // just write the type
                writer.WriteAttributeString("Type", EncodeString(o.GetType().FullName));
            }
          } else {  // this is a reference to an object - enqueue it
            // compute an ID for this object if it doesn't already have one
            if (dObjectIDs.ContainsKey(o) == false) {  // enqueue for writing
              int iID = dObjectIDs.Keys.Count;
              dObjectIDs.Add(o, iID);
              lstWriteQueue.Add(o);
            }
            writer.WriteStartElement("Reference");
            if (strName.Length > 0)  // (the object might only have a name if we're writing a reference to it)
              writer.WriteAttributeString("Name", EncodeString(strName));
            writer.WriteAttributeString("ID", dObjectIDs[o].ToString());
          }
          writer.WriteEndElement();
        }
      } catch (Exception ex) {
        MessageBox.Show(ex.Message);
      }
    }

    private void WriteList(IList lstObjects) {
      writer.WriteStartElement("List");
      writer.WriteAttributeString("ID", dObjectIDs[lstObjects].ToString());
      // this next part is tricky: we need to find out the type of object stored in this IList. The only way to do this is to parse the GetType().FullName.
      string strSubtype = lstObjects.GetType().FullName;
      strSubtype = strSubtype.Substring(strSubtype.IndexOf("[[") + 2);
      strSubtype = strSubtype.Substring(0, strSubtype.IndexOf(","));
      writer.WriteAttributeString("Type", EncodeString(strSubtype));
      // write contained objects
      for (int i = 0; i < lstObjects.Count; i++)
        WriteObject(lstObjects[i]);
    }

    private void WriteArray(IList lstObjects) {
      writer.WriteStartElement("Array");
      writer.WriteAttributeString("ID", dObjectIDs[lstObjects].ToString());
      // this next part is tricky: we need to find out the type of object stored in this array. The only way to do this is to parse the GetType().FullName.
      string strSubtype = lstObjects.GetType().FullName;
      strSubtype = strSubtype.Substring(0, strSubtype.Length - 2);  // strip off the indexer at the end
      writer.WriteAttributeString("Type", EncodeString(strSubtype));
      // write contained objects
      for (int i = 0; i < lstObjects.Count; i++)
        WriteObject(lstObjects[i]);
    }

    private void WriteDictionary(IDictionary dictionary) {
      writer.WriteStartElement("Dictionary");
      writer.WriteAttributeString("ID", dObjectIDs[dictionary].ToString());
      // this next part is tricky: we need to find out the type of object stored in this array. The only way to do this is to parse the GetType().FullName.
      string strSubtype = dictionary.GetType().FullName;
      string strSubtypeKey = strSubtype.Substring(strSubtype.IndexOf("[[") + 2);
      strSubtypeKey = strSubtypeKey.Substring(0, strSubtypeKey.IndexOf(","));
      string strSubtypeValue = strSubtype.Substring(strSubtype.IndexOf("],[") + 3);
      strSubtypeValue = strSubtypeValue.Substring(0, strSubtypeValue.IndexOf(","));
      writer.WriteAttributeString("Type", EncodeString(strSubtypeKey + "," + strSubtypeValue));
      foreach (object oKey in dictionary.Keys) {
        WriteObject(oKey);
        WriteObject(dictionary[oKey]);
      }
    }

    #endregion

    #region Helper Functions

    /// <summary>
    /// Write the specified object to an XMLConfig file. (This function should be invoked in the WriteObject() method
    /// of the object in order to write each of its members.)
    /// </summary>
    /// <param name="strName">The name to be assigned to the object in the XMLConfig file.</param>
    /// <param name="o">The object to be written.</param>
    public void WriteObject(string strName, object o) {
      // This function is here to allow IWritable classes to invoke it with a ("member name", object) syntax - and to prevent them from invoking bWrite.
      WriteObject(o, strName, false);
    }

    private static string BinarySerialize(object o) {
      try {
        MemoryStream memoryStream = new MemoryStream();
        new BinaryFormatter().Serialize(memoryStream, o);
        return System.Convert.ToBase64String(memoryStream.ToArray());
      } catch (Exception ex) {
        MessageBox.Show("Error serializing object: " + ex.Message);
        return "";
      }
    }

    /// <summary>
    /// Encrypts the specified cleartext string with the specified password using the Rijndael encryption method.
    /// </summary>
    /// <param name="strClearText">The cleartext string to be encrypted.</param>
    /// <param name="strPassword">The password to be used to encrypt the cleartext string.</param>
    public static string EncryptString(string strClearText, string strPassword) {
      byte[] bClearBytes = System.Text.Encoding.Unicode.GetBytes(strClearText); 
      System.Security.Cryptography.PasswordDeriveBytes pdb = new System.Security.Cryptography.PasswordDeriveBytes(strPassword, new byte[] { 0x49, 0x76, 0x61, 0x6e, 0x20, 0x4d, 0x65, 0x64, 0x76, 0x65, 0x64, 0x65, 0x76 });
      System.IO.MemoryStream ms = new System.IO.MemoryStream();
      System.Security.Cryptography.Rijndael alg = System.Security.Cryptography.Rijndael.Create();
      alg.Key = pdb.GetBytes(32);
      alg.IV = pdb.GetBytes(16);
      System.Security.Cryptography.CryptoStream cs = new System.Security.Cryptography.CryptoStream(ms, alg.CreateEncryptor(), System.Security.Cryptography.CryptoStreamMode.Write);
      cs.Write(bClearBytes, 0, bClearBytes.Length);
      cs.Close();
      return Convert.ToBase64String(ms.ToArray());
    }

    private static string EncodeString(string s) {
      return System.Web.HttpUtility.HtmlEncode(s);
    }
    
    private void Close() {
      if (writer != null) {
        writer.Close();
        writer = null;
      }
    }

    #endregion

  }

}
